package com.example.android_example;

import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

import android.app.Activity;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;

import com.aliyun.mbaas.oss.OSSClient;
import com.aliyun.mbaas.oss.callback.SaveCallback;
import com.aliyun.mbaas.oss.model.AccessControlList;
import com.aliyun.mbaas.oss.model.OSSException;
import com.aliyun.mbaas.oss.model.TokenGenerator;
import com.aliyun.mbaas.oss.storage.OSSBucket;
import com.aliyun.mbaas.oss.storage.OSSData;
import com.aliyun.mbaas.oss.storage.OSSFile;
import com.aliyun.mbaas.oss.util.OSSLog;
import com.aliyun.mbaas.oss.util.OSSToolKit;

public class MainActivity extends Activity {

    static final String accessKey = "sk"; // 测试代码没有考虑AK/SK的安全性
    static final String screctKey = "ak";

    public OSSBucket sampleBucket;

    static {
        OSSClient.setGlobalDefaultTokenGenerator(new TokenGenerator() { // 设置全局默认加签器
            @Override
            public String generateToken(String httpMethod, String md5, String type, String date,
                    String ossHeaders, String resource) {

                String content = httpMethod + "\n" + md5 + "\n" + type + "\n" + date + "\n" + ossHeaders
                        + resource;

                return OSSToolKit.generateToken(accessKey, screctKey, content);
            }
        });
        OSSClient.setGlobalDefaultACL(AccessControlList.PUBLIC_READ); // 设置全局默认bucket访问权限

        /*
         * 如果需要采用服务端加签的方式，可以直接在tokenGenerator中向你的业务服务器发起
         * 同步http post请求，把相关字段拼接之后发过去，然后获得加签结果。
         *
         *
        OSSClient.setGlobalDefaultTokenGenerator(new TokenGenerator() {
            @Override
            public String generateToken(String httpMethod, String md5, String type, String date,
                    String ossHeaders, String resource) {

                String content = httpMethod + "\n" + md5 + "\n" + type + "\n" + date + "\n" + ossHeaders
                        + resource;

                List<NameValuePair> params = new ArrayList<NameValuePair>();
                params.add(new BasicNameValuePair("content", content));
                HttpPost post = new HttpPost("http://localhost/oss");
                String sign = null;

                try {
                    post.setEntity(new UrlEncodedFormEntity(params));
                    HttpResponse response = new DefaultHttpClient().execute(post);
                    sign = EntityUtils.toString(response.getEntity()).trim();
                } catch (Exception ignore) {
                }
                Log.d("OSS_Test", "[genToken] - remote: " + sign);
                return sign;
            }
        });

         ********************************************************
         * 以下是加签服务器的代码示例： (nginx + lua脚本)
         *
         * local access_key = "ak";
         * local screct_key = "sk";
         * local sign_str;
         *
         * ngx.req.read_body();
         * --local body = ngx.req.get_body_data();
         *
         * local args, err = ngx.req.get_post_args();
         * for key, val in pairs(args) do
         *         if key == "content" then
         *                 sign_str = val;
         *         end
         * end
         *
         * local sign_result = ngx.encode_base64(ngx.hmac_sha1(screct_key, sign_str));
         *
         * ngx.say("OSS "..access_key..":"..sign_result);
         ********************************************************/
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        OSSLog.enableLog(true);

        OSSClient.setApplicationContext(getApplicationContext()); // 传入应用程序context

        // 开始单个Bucket的设置
        sampleBucket = new OSSBucket("android-test");
        sampleBucket.setBucketHostId("oss-cn-hangzhou.aliyuncs.com"); // 可以在这里设置数据中心域名或者cname域名
        //sampleBucket.setBucketACL(AccessControlList.PUBLIC_READ_WRITE); // 如果这个Bucket跟全局默认的权限设置不一致
        sampleBucket.setBucketACL(AccessControlList.PRIVATE);
        // sampleBucket.setBucketHostId("oss-cn-hangzhou.aliyuncs.com"); // 如果这个Bucket跟全局默认的数据中心不一致，就需要单独设置
        // sampleBucket.setBucketTokenGen(new TokenGenerator() {...}); // 如果这个Bucket跟全局默认的加签方法不一致，就需要单独设置


        Button startDownload = (Button) findViewById(R.id.button1);
        startDownload.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    doUpload();
                    doUploadFile();
                    doDownload();
                } catch (Exception e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }
            }
        });

    }

    public void doDownload() throws Exception {
        Runnable r = new Runnable() {
            @Override
            public void run() {
                OSSData ossData = new OSSData(sampleBucket, "hello.txt");
                try {
                    ossData.get();
                } catch (OSSException e) {
                    // TODO Auto-generated catch block
                    e.printStackTrace();
                }

            }
        };
        new Thread(r).start();
    }

    public void doUpload() throws Exception {
        try {
            OSSData ossData2 = new OSSData(sampleBucket, "test-upload");
            ossData2.setData("upload this piece of data".getBytes(), "text/txt");

            ossData2.uploadInBackground(new SaveCallback() {
                @Override
                public void onProgress(String arg0, int arg1, int arg2) {
                }
                @Override
                public void onFailure(String arg0, OSSException arg1) {
                }
                @Override
                public void onSuccess(String arg0) {
                }
            });
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void doUploadFile() throws Exception {
        OSSFile ossFile = new OSSFile(sampleBucket, "test.jpg");
        ossFile.setUploadFilePath("/storage/sdcard0/src_file/test.jpg", "image/jpg");
        ossFile.uploadInBackground(new SaveCallback() {

            @Override
            public void onProgress(String arg0, int arg1, int arg2) {
                // TODO Auto-generated method stub
            }

            @Override
            public void onFailure(String arg0, OSSException arg1) {
                // TODO Auto-generated method stub
            }
            @Override
            public void onSuccess(String arg0) {
                // TODO Auto-generated method stub
            }
        });
    }
}
